#include <string>
#include <list>

#include "QMATH_Evaluator.H"

#include "QMATH_Expression.H"

// public:
ostream &operator<< (ostream &output, QMATH_Expression& exp)
{
  //output << "Result:";
  exp.Evaluate();
  return output;
}

QMATH_Expression::QMATH_Expression ()
{
  basic_object = NULL;
  Type(none);
  if (!p_evaluator)
    p_evaluator = new QMATH_Evaluator();// Use default evaluator.
}

QMATH_Expression::QMATH_Expression (int num_args, ...)
{
  basic_object = NULL;
  Type(none);
  if (!p_evaluator)
    p_evaluator = new QMATH_Evaluator();// Use default evaluator.

  va_list arg_list;
  int arg_index;

  va_start(arg_list, num_args);
  for (arg_index = 1; arg_index <= num_args; ++arg_index)
    {
      Add_item(va_arg(arg_list, QMATH_Expression*));
    }
  va_end(arg_list);
}

QMATH_Expression::QMATH_Expression (const c_string new_string)
{
  basic_object = new char[strlen(new_string) + 1];
  Type(none);
  strcpy(basic_object, new_string);
  if (!p_evaluator)
    p_evaluator = new QMATH_Evaluator();// Use default evaluator.
}

QMATH_Expression* QMATH_Expression::Type (om_object_type_t new_object_type)
{
  object_type = new_object_type;
}

om_object_type_t QMATH_Expression::Type ()
{
  return object_type;
}

char* QMATH_Expression::Basic_object ()
{
  return basic_object;
}

QMATH_Expression &QMATH_Expression::Object (int object_index)
{
  list<QMATH_Expression>::iterator object_iterator;
  object_iterator = construct.begin();
  for (int index = 0; index < object_index; ++index)
    ++object_iterator;
  return *object_iterator;
}

int QMATH_Expression::Object_count ()
{
  // Returns 0 for basic objects.
  return construct.size();
}

bool QMATH_Expression::Symbol (string* p_value)
{
  if (!basic_object) return false;
  *p_value = basic_object;
  return true;
}

void QMATH_Expression::Evaluate ()
{
  p_evaluator->Apply(*this);
}

bool QMATH_Expression::Select_evaluator (c_string new_evaluator_module)
{
  p_evaluator->Select(new_evaluator_module);
  return true;
}

//private:
P_QMATH_Evaluator QMATH_Expression::p_evaluator = NULL;

bool QMATH_Expression::Add_item (QMATH_Expression* p_new_item)
{
  if (!p_new_item) return false;// Refuse to add a NULL expression.

  // Convert atom to unary list, so that we can add elements to it.
  if (basic_object)
    {
      QMATH_Expression* p_new_item_for_atom;
      p_new_item_for_atom = new QMATH_Expression(basic_object);
      construct.push_back(*p_new_item_for_atom);
      delete [] basic_object;
      basic_object = NULL;
    }
  // Add item at list tail.
  if (p_new_item->basic_object)
    construct.push_back(*p_new_item);
  else
    construct.push_back(*p_new_item);
  return true;
}

bool QMATH_Expression::Add_item (const c_string new_string)
{
  if (!new_string) return false;// Refuse to add a NULL expression.

  QMATH_Expression* p_new_item;
  // Convert atom to unary list, so that we can add elements to it.
  if (basic_object)
    {
      QMATH_Expression* p_new_item_for_atom;
      p_new_item_for_atom = new QMATH_Expression(basic_object);
      construct.push_back(*p_new_item_for_atom);
      delete [] basic_object;
      basic_object = NULL;
    }
  p_new_item = new QMATH_Expression(new_string);
  // Add item at list tail.
  construct.push_back(*p_new_item);
  return true;
}
