// Evaluator_*.cc -- These functions are used only from QMATH_Evaluator_loader.


#include "../QMATH_Evaluator.H"

#include "Evaluator_mathml.H"

void Evaluator_mathml_Operator_default (QMATH_Expression &exp);
void Evaluator_mathml_Operator_Plus (QMATH_Expression &exp);
void Evaluator_mathml_Operator_Minus (QMATH_Expression &exp);
void Evaluator_mathml_Operator_Plus_minus (QMATH_Expression &exp);
void Evaluator_mathml_Operator_Times (QMATH_Expression &exp);
void Evaluator_mathml_Operator_Divide (QMATH_Expression &exp);
void Evaluator_mathml_Operator_Power (QMATH_Expression &exp);
void Evaluator_mathml_Operator_Root (QMATH_Expression &exp);
void Evaluator_mathml_Operator_Equals (QMATH_Expression &exp);

//  This is the public interface.
void Evaluator_mathml_Register (evaluator_table_entry &evaluator_entry)
{
  operator_table_entry operator_entry;
  evaluator_entry.name = "mathml";

  //  The default operator must be always present.
  operator_entry.name = "default";
  operator_entry.code = &Evaluator_mathml_Operator_default;
  evaluator_entry.operator_table.push_back(operator_entry);

  //  Here we register the rest of the operators.

  operator_entry.name = "Plus";
  operator_entry.code = &Evaluator_mathml_Operator_Plus;
  evaluator_entry.operator_table.push_back(operator_entry);
  
  operator_entry.name = "Minus";
  operator_entry.code = &Evaluator_mathml_Operator_Minus;
  evaluator_entry.operator_table.push_back(operator_entry);
  
  operator_entry.name = "Plus_minus";
  operator_entry.code = &Evaluator_mathml_Operator_Plus_minus;
  evaluator_entry.operator_table.push_back(operator_entry);
  
  operator_entry.name = "Times";
  operator_entry.code = &Evaluator_mathml_Operator_Times;
  evaluator_entry.operator_table.push_back(operator_entry);
  
  operator_entry.name = "Divide";
  operator_entry.code = &Evaluator_mathml_Operator_Divide;
  evaluator_entry.operator_table.push_back(operator_entry);
  
  operator_entry.name = "Power";
  operator_entry.code = &Evaluator_mathml_Operator_Power;
  evaluator_entry.operator_table.push_back(operator_entry);
  
  operator_entry.name = "Root";
  operator_entry.code = &Evaluator_mathml_Operator_Root;
  evaluator_entry.operator_table.push_back(operator_entry);
  
  operator_entry.name = "Equals";
  operator_entry.code = &Evaluator_mathml_Operator_Equals;
  evaluator_entry.operator_table.push_back(operator_entry);
  

}

//  The following function is only known to Evaluator_*_Opertor_*().
//  It seems that "const c_string" is considered just as "char*" instead
// of "const char*". I don't know why. But it was faster to circumvent it
// than finding the cause.
const char* Evaluator_mathml_Translate_atom (const char* basic_object)
{
  #include "Evaluator_mathml_atom_dictionary.icc"
  int index;
  index = 0;
  while (strcmp("", atom_dictionary[index][0]) != 0)
    {
      if (strcmp(basic_object, atom_dictionary[index][0]) == 0)
        break;
      ++index;
    }
  if (strcmp("", atom_dictionary[index][1]) == 0)
    return basic_object;
  else
    return atom_dictionary[index][1];
}

//  The actual code for every operator.
//  The following functions are only known to the *_Register() one.
//  If you want an operator to be an alias for another, do it in
// the *_Register() function above. It will be much easier to maintain.
void Evaluator_mathml_Operator_default (QMATH_Expression &exp)
{
  char* begin_object;
  char* end_object;
  
  switch (exp.Type())
    {
    case none:
      begin_object = " NONE.";
      end_object   = "";
      break;
    case integer:
      begin_object = " <mn>";
      end_object   = "</mn>";
      break;
    case fp_number:
      begin_object = " <mn>";
      end_object   = "</mn>";
      break;
    case char_string:
      begin_object = " <mt>";
      end_object   = "</mt>";
      break;
    case byte_array:
      begin_object = " <mt>";
      end_object   = "</mt>";
      break;
    case symbol:
      begin_object = " <mo>";
      end_object   = "</mo>";
      break;
    case variable:
      begin_object = " <mi>";
      end_object   = "</mi>";
      break;
    case application:
      begin_object = " <mrow>";
      end_object   = "</mrow>";
      break;
    case attribution:
      begin_object = " <mrow>";
      end_object   = "</mrow>";
      break;
    case binding:
      begin_object = " <mrow>";
      end_object   = "</mrow>";
      break;
    case error:
      begin_object = " <error>";
      end_object   = " </error>";
      break;
    case math:
      begin_object = " <math>";
      end_object   = " </math>";
      break;
    default:
      begin_object = " UNKNOWN";
      end_object   = "";
      break;
    }
  
  cout << begin_object;
  
  if (exp.Object_count() == 0)
    {
      cout << exp.Basic_object();
    }
  else
    {
      for (int index = 0; index < exp.Object_count(); ++index)
        exp.Object(index).Evaluate();
    }

  cout << end_object;
}

void Evaluator_mathml_Operator_Plus (QMATH_Expression &exp)
{
  cout << " <mrow>";
  
  for (int index = 1; index < exp.Object_count(); ++index)
    {
      exp.Object(index).Evaluate();
      if (index+1 < exp.Object_count())
        cout << " <mo>+</mo>";
    }

  cout << " </mrow>";
}

void Evaluator_mathml_Operator_Minus (QMATH_Expression &exp)
{
  cout << " <mrow>";
  
  if (exp.Object_count() == 2)
    cout << "<mo>-</mo>";// Unary minus.
  
  for (int index = 1; index < exp.Object_count(); ++index)
    {
      exp.Object(index).Evaluate();
      if (index+1 < exp.Object_count())
        cout << " <mo>-</mo>";
    }

  cout << " </mrow>";
}

void Evaluator_mathml_Operator_Plus_minus (QMATH_Expression &exp)
{
  cout << " <mrow>";
  
  if (exp.Object_count() == 2)
    cout << "<mo>&PlusMinus;</mo>";// Unary plus_minus.
  
  for (int index = 1; index < exp.Object_count(); ++index)
    {
      exp.Object(index).Evaluate();
      if (index+1 < exp.Object_count())
        cout << " <mo>&PlusMinus;</mo>";
    }

  cout << " </mrow>";
}

void Evaluator_mathml_Operator_Times (QMATH_Expression &exp)
{
  cout << " <mrow>";
  
  for (int index = 1; index < exp.Object_count(); ++index)
    {
      exp.Object(index).Evaluate();
      if (index+1 < exp.Object_count())
        cout << " <mo>&InvisibleTimes;</mo>";
    }

  cout << " </mrow>";
}

void Evaluator_mathml_Operator_Divide (QMATH_Expression &exp)
{
  cout << " <mfrac><mrow>";
  
  for (int index = 1; index < exp.Object_count(); ++index)
    {
      exp.Object(index).Evaluate();
      if (index+1 < exp.Object_count())
        cout << " </mrow><mrow>";
    }

  cout << " </mrow></mfrac>";
}

void Evaluator_mathml_Operator_Power (QMATH_Expression &exp)
{
  cout << " <msup><mrow>";
  
  for (int index = 1; index < exp.Object_count(); ++index)
    {
      exp.Object(index).Evaluate();
      if (index+1 < exp.Object_count())
        cout << " </mrow><mrow>";
    }

  cout << " </mrow></msup>";
}

void Evaluator_mathml_Operator_Root (QMATH_Expression &exp)
{
  QMATH_Expression* p_exp;
  p_exp = NULL;
  
  if (exp.Object(1).Type() == application)
    if (strcmp(exp.Object(1).Object(0).Basic_object(), ",") == 0)
      p_exp = &(exp.Object(1));
  
  if (p_exp == NULL) p_exp = &exp;

  if (p_exp->Object_count())
    {
      cout << " <mroot><mrow>";
      if (p_exp == &exp)
        {
          for (int index = 1; index < p_exp->Object_count(); ++index)
            {
              p_exp->Object(index).Evaluate();
              if (index+1 < exp.Object_count())
                cout << " </mrow><mrow>";
            }
        }
      else
        {
          for (int index = p_exp->Object_count() -1; index >= 1; --index)
            {
              p_exp->Object(index).Evaluate();
              if (index-1 >= 1)
                cout << " </mrow><mrow>";
            }
        }
      cout << " </mrow></mroot>";
    }
  else
    {
      cout << " <msqrt><mrow>";
      p_exp->Object(1).Evaluate();
      cout << " </mrow></msqrt>";
    }
}

void Evaluator_mathml_Operator_Equals (QMATH_Expression &exp)
{
  cout << " <mrow>";
  
  for (int index = 1; index < exp.Object_count(); ++index)
    {
      exp.Object(index).Evaluate();
      if (index+1 < exp.Object_count())
        cout << " </mrow><mo>=</mo><mrow>";
    }

  cout << " </mrow>";
}

