#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <stddef.h>

#include "mathml.H"

char* entity[][2] = {
  { "alpha",     "&alpha;"     },
  { "Alpha",     "&Alpha;"     },
  { "beta",      "&beta;"      },
  { "Beta",      "&Beta;"      },
  { "gamma",     "&gamma;"     },
  { "Gamma",     "&Gamma;"     },
  { "phi",       "&phi;"       },
  { "Phi",       "&Phi;"       },
  { "psi",       "&psi;"       },
  { "Psi",       "&Psi;"       },
  { "pi",        "&pi;"        },
  { "Pi",        "&Pi;"        },
  { "xi",        "&xi;"        },
  { "Xi",        "&Xi;"        },
  { "chi",       "&chi;"       },
  { "Chi",       "&Chi;"       },
  { "theta",     "&theta;"     },
  { "Theta",     "&Theta;"     },
  { "omega",     "&omega;"     },
  { "Omega",     "&Omega;"     },
  { "delta",     "&delta;"     },
  { "Delta",     "&Delta;"     },
  { "aleph",     "</math>&alephsym;<math>"},
  { "inf",       "</math>&infin;<math>"},
  { "->",        "&DoubleRightArrow;"},
  { "<-",        "&DoubleLeftArrow;"},
  { "<->",       "&DoubleLeftRightArrow;"},
  { "ptodo",     "&ForAll;"    },
  { "existe",    "&Exists;"    },
  { "tq",        "|"           },
  { "=",         "="           },
  { "=",        "&NotEqual;"  },
  { "+-",        "&PlusMinus;" },
  { "pert",      "&Element;"   },
  { "pert",     "&NotElement;"},
  { "cont",      "&Superset;"  },
  { "icont",     "&SupersetEqual;"},
  { "subcon",    "&Subset;"    },
  { "isubcon",   "&SubsetEqual"},
  { "unin",     "&Union;"     },
  { "inter",     "&Intersection"},
  { "vaco",     "</math>&Oslash;<math>"},
  { "\\",        "\\"          },
  { "...",       "&TripleDot;" },
  { "~",         "&Congruent;" },
  { ">=",        "&GreaterEqual;"},
  { "<=",        "&le;"        },
  { ">",         "&gt;"        },
  { "<",         "&lt;"        },
  { "",         "</math>&bull;<math>"},
  { "prop",      "&Proportional;"},
  { "neutro",    "e"           },
  { "csuma",     "&CirclePlus;"},
  { "cprod",     "&CircleTimes;"},
  { "",          ""            }
};

format_table_entry output_format_mathml[] =
{
  {"/",   "<mfrac><mrow>", "</mrow><mrow>",      "</mrow></mfrac>" },
  {"+",   "",              "<mo>+</mo>",         ""                },
  {"-",   "",              "<mo>-</mo>",         ""                },
  {"*",   "",              "<mo>*</mo>",         ""                },
  {"^",   "<msup><mrow>",  "</mrow><mrow>",      "</mrow></msup>"  },
  {"**",  "<msup><mrow>",  "</mrow><mrow>",      "</mrow></msup>"  },
  {"//",  "<mroot><mrow>", "</mrow><mrow>",      "</mrow></mroot>" },
  {"\\*", "",              "<mo></mo>",         ""                },
  {"|",   "",              "&Or;",               ""                },
  {"",   "",              "&deg;",              ""                },
  {"",   "",              "&Not;",              ""                },
  {"|",   "",              "&Or;",               ""                },
  {"&",   "",              "&And;",              ""                },
  {",",   "",              "<mo>,</mo>",         ""                },
  {"+",   "",              "<mo>+</mo>",         ""                },
  {"+",   "",              "<mo>+</mo>",         ""                },
  {"+",   "",              "<mo>+</mo>",         ""                },
  {"+",   "",              "<mo>+</mo>",         ""                },
  {"+",   "",              "<mo>+</mo>",         ""                },
  {"+",   "",              "<mo>+</mo>",         ""                },
  {"",    "",              "",                   ""                }
};

c_string QMATH_Processor::Translate_atom (c_string basic_object)
{
  int index;
  index = 0;
  while (strcmp("", entity[index][0]) != 0)
    {
      if (strcmp(basic_object, entity[index][0]) == 0)
        break;
      ++index;
    }
  if (strcmp("", entity[index][1]) == 0)
    return basic_object;
  else
    return entity[index][1];
}

void QMATH_Processor::begin_math(void)
{
  if (!inside_math)
    {
      fprintf(stdout, "<math>");
      inside_math = 1;
    }
}

void QMATH_Processor::end_math(void)
{
  if (inside_math)
    {
      fprintf(stdout, "</math>");
	inside_math = 0;
    }
}
