// Evaluator_*.cc -- These functions are used only from QMATH_Evaluator_loader.
!$name="default";
!@function_list=(  'And', 'Or', 'Xor', 'Not', 'Abs', 'Conjugate', 'Inverse', 'Ident', 'Plus', 'Minus', 'Times', 'Divide', 'Factorial', 'Exp', 'Ln', 'Log', 'Sin', 'Cos', 'Tan', 'Sec', 'Csc', 'Cot', 'Sinh', 'Cosh', 'Tanh', 'Sech', 'Csch', 'Coth', 'Arcsin', 'Arccos', 'Arctan', 'Quotient', 'Power', 'Rem', 'Max', 'Min', 'Gcd', 'Mean', 'Sdev', 'Variance', 'Median', 'Mode', 'Interval', 'Set', 'Vector', 'Declare', 'Determinant', 'Transpose', 'Implies', 'Setdiff', 'Selector', 'Union', 'Intersect', 'Fn', 'Compose', 'Int', 'Sum', 'Product', 'Diff', 'Partialdiff', 'Forall', 'Exists', 'Root');
  
#include "../QMATH_Evaluator.H"

#include "Evaluator_$name.H"

void Evaluator_$name\_Operator_default (QMATH_Expression &exp);
!foreach $func (@function_list) {
void Evaluator_$name\_Operator_$func (QMATH_Expression &exp);
!}

//  This is the public interface.
void Evaluator_$name\_Register (evaluator_table_entry &evaluator_entry)
{
  operator_table_entry operator_entry;
  evaluator_entry.name = "$name";

  //  The default operator must be always present.
  operator_entry.name = "default";
  operator_entry.code = &Evaluator_$name\_Operator_default;
  evaluator_entry.operator_table.push_back(operator_entry);

  //  Here we register the rest of the operators.

!#!foreach $func (@function_list) {
!#  operator_entry.name = "$func";
!#  operator_entry.code = \&Evaluator_$name\_Operator_$func;
!#  evaluator_entry.operator_table.push_back(operator_entry);
!#  
!#!}

}

//  The following function is only known to Evaluator_*_Opertor_default().
//  It seems that "const c_string" is considered just as "char*" instead
// of "const char*". I don't know why. But it was faster to circumvent it
// than finding the cause.
const char* Evaluator_$name\_Translate_atom (const char* basic_object)
{
  #include "Evaluator_$name\_atom_dictionary.icc"
  int index;
  index = 0;
  while (strcmp("", atom_dictionary[index][0]) != 0)
    {
      if (strcmp(basic_object, atom_dictionary[index][0]) == 0)
        break;
      ++index;
    }
  if (strcmp("", atom_dictionary[index][1]) == 0)
    return basic_object;
  else
    return atom_dictionary[index][1];
}

extern int nest_level;
//  The actual code for every operator.
//  The following functions are only known to the previous, *_Register().
//  If you want an operator to be an alias for another, do it in
// the *_Register() function above. It will be much easier to maintain.
void Evaluator_$name\_Operator_default (QMATH_Expression &exp)
{
  char* begin_object;
  char* end_object;
  
  switch (exp.Type())
    {
    case none:        begin_object = " NONE.";     end_object = "";      break;
    case integer:     begin_object = " INT.";      end_object = "";      break;
    case fp_number:   begin_object = " FP.";       end_object = "";      break;
    case char_string: begin_object = " STR.";      end_object = "";      break;
    case byte_array:  begin_object = " ARRAY.";    end_object = "";      break;
    case symbol:      begin_object = " SYM.";      end_object = "";      break;
    case variable:    begin_object = " VAR.";      end_object = "";      break;
    case application: begin_object = " APP(";      end_object = " )";    break;
    case attribution: begin_object = " ATT(";      end_object = " )";    break;
    case binding:     begin_object = " BIND(";     end_object = " )";    break;
    case error:       begin_object = " ERR(";      end_object = " )";    break;
    case math:        begin_object = " MATH(";     end_object = " )";    break;
    default:          begin_object = " UNKNOWN"; end_object = "";      break;
    }

  cout << begin_object;
  
  if (exp.Object_count() == 0)
    {
      cout << exp.Basic_object();
    }
  else
    {
      for (int index = 0; index < exp.Object_count(); ++index)
        exp.Object(index).Evaluate();
    }

  cout << end_object;
}

!#!foreach $func (@function_list) {
!#void Evaluator_$name_$func (list<string> &accumulator, string &result)
!#  {
!#    default(accumulator, result);
!#  }
!#!}
