#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <locale.h>
extern FILE* yyin;
extern FILE* yyout;
char* input_file_name = NULL;
char* output_file_name = NULL;
char* file_write_mode = NULL;
char* qmath_evaluator_module = "default";
int yyparse(void);
//#include "actions.H"
//#include "mathml.H"
#include "QMATH_Expression.H"
#include "qmath.tab.h"
extern int yydebug;

#include "qmath.H"

int main(int argc, char *argv[])
  {
    setlocale(LC_ALL, "");
    file_write_mode = "wb";
    if (argc > 1)
      {
        int index;
        for (index = 1; index <argc; ++index)
          {
            if (strcmp(argv[index], "-h") == 0
                || strcmp(argv[index], "--help") == 0)
              {
                print_help(argv[0]);
                return(0);
              }
            else if (strcmp(argv[index], "-a") == 0)
              {
                file_write_mode = "ab";
              }
            else if (strcmp(argv[index], "-d") == 0)
              {
                yydebug=1;
              }
            else if (strcmp(argv[index], "-o") == 0)
              {
                if (argc > index) {
                  output_file_name = argv[index + 1];
                  ++index;
                }
              }
            else if (strcmp(argv[index], "-e") == 0)
              {
                if (argc > index) {
                  qmath_evaluator_module = argv[index + 1];
                  ++index;
                }
              }
            else if (*argv[index] == '-')
              {
                fprintf(stderr, "Unknown option '%s'. Use %s -h for help.\n", argv[index], argv[0]);
                return 1;
              }
            else if (!input_file_name)
              {
                input_file_name = argv[index];
              }
            else if (!output_file_name)
              {
                output_file_name = argv[index];
              }
            else
              {
                fprintf(stderr, "Too many inputs. Use %s -h for help.\n", argv[0]);
                return 1;
              }
          }
	if (input_file_name && output_file_name)
          {
            if (strcmp(input_file_name, output_file_name) == 0)
              {
                fprintf(stderr, "Can not write on input file.\n");
                return 1;
              }
          }
        if (input_file_name)
          {
            fprintf(stderr, "Processing file: %s\n", input_file_name);
            if (!(yyin = fopen( input_file_name, "rb" )))
              {
                fprintf(stderr, "Cannot open file for reading: %s\n", input_file_name);
                if (output_file_name) fclose(yyout);
                return 1;
              }
          }
        if (output_file_name)
          {
            if (!(yyout = fopen( output_file_name, file_write_mode )))
              {
                fprintf(stderr, "Cannot open file for writing: %s\n", output_file_name);
                return 1;
              }
          }
      }
    if (strcmp("mathml", qmath_evaluator_module) == 0)
      {
        if (yyout) fprintf(yyout,  "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\"\n   \"http://www.w3.org/TR/REC-html40/loose.dtd\">\n<html><head><title>QMathML</title></head>\n<body>\n\n");
        else       fprintf(stdout, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\"\n   \"http://www.w3.org/TR/REC-html40/loose.dtd\">\n<html><head><title>QMathML</title></head>\n<body>\n\n");
      }

    yyparse();//bison
    //yylex();//flex

    if (strcmp("mathml", qmath_evaluator_module) == 0)
      {
        if (yyout) fprintf(yyout,  "\n\n</body>\n</html>");
        else       fprintf(stdout, "\n\n</body>\n</html>");
      }
    //fprintf(stderr, "Total source lines: %d\n", current_line);
    if (output_file_name) fclose(yyout);
    if (input_file_name) fclose(yyin);
    return 0;
  }

void print_help(char* command_name)
  {
    fprintf(stderr, "%s Written 1999 Alberto Gonzlez Palomo\n\n", command_name);
    fprintf(stderr, "Usage: %s [-e <evaluator_module>] [<input file>]\n", command_name);
    fprintf(stderr, "    -e   The evaluator module to be applied to the input. It is one of:\n");
    fprintf(stderr, "           default\n");
    fprintf(stderr, "           mathml\n");
    fprintf(stderr, "         The default is, you guess it, 'default'.\n");
    fprintf(stderr, "    If no input file name is given, the program uses stdin.\n");
  }
