%option noyywrap
%option nomain

%{

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <locale.h>
#include "QMATH_Expression.H"
#include "qmath.tab.h"
#include "input_translation_table.icc"

  char* qmath_lang = "";
  
  int yyparse(void);

%}


MATH_BEGIN			"$"
MATH_END			"$"
QMATH_DOC_BEGIN                 "QMATH "[0-9]+("."[0-9]+)+" LANG="
IDENTIFIER			[a-zA-Z]+'*
INT_NUMBER                      [0-9]+
FP_NUMBER			[0-9]+([.'][0-9]+)
CHAR_STRING			\"([^"]*"\\\"")*\"
OPERATOR_PLUS                   "+"|"-"|"+-"
OPERATOR_PROD                   "*"|"/"
OPERATOR_EXP                    "^"|"**"|"//"
OPERATOR_EQ                     "="|":="
FUNCTION                        {IDENTIFIER}"("
SEPARATOR_COMMA			","
SEPARATOR_COLON                 ":"
SEPARATOR_SEMICOLON             ";"

%x QMATH
%x LANG_SPEC
%x MATH

%%

<INITIAL>^{QMATH_DOC_BEGIN} BEGIN(LANG_SPEC);
<LANG_SPEC>[a-z]+       {
                          qmath_lang = new char[strlen(yytext)+1];
                          strcpy(qmath_lang, yytext);
                          BEGIN(QMATH);
                        }
<QMATH>{MATH_BEGIN}     {
			  //begin_math();
			  BEGIN(MATH);
			}
<MATH>{MATH_BEGIN}	{
			  //sethtml(); Done in qmath.y/yyparse()
			  BEGIN(QMATH);
			  return '\n';
			}
<MATH>\n		// Eat carriage returns
<MATH>[ \t]             // Eat blanks
<MATH>{IDENTIFIER}	{yylval=(int) new QMATH_Expression(translate_input_symbol(yytext, qmath_lang)); return ID;}
<MATH>{FUNCTION}	{*(yytext+strlen(yytext)-1) = 0; yylval=(int) new QMATH_Expression(translate_input_symbol(yytext, qmath_lang)); return FUNC;}
<MATH>{INT_NUMBER}	{yylval=(int) new QMATH_Expression(translate_input_number(yytext, qmath_lang)); return INT_NUM;}
<MATH>{FP_NUMBER}	{yylval=(int) new QMATH_Expression(translate_input_number(yytext, qmath_lang)); return FP_NUM;}
<MATH>{CHAR_STRING}	{yylval=(int) new QMATH_Expression(translate_input_number(yytext, qmath_lang)); return STRING;}
<MATH>{OPERATOR_PLUS}	{yylval=(int) new QMATH_Expression(translate_input_symbol(yytext, qmath_lang)); return OP_PLUS;}
<MATH>{OPERATOR_PROD}	{yylval=(int) new QMATH_Expression(translate_input_symbol(yytext, qmath_lang)); return OP_PROD;}
<MATH>{OPERATOR_EXP}	{yylval=(int) new QMATH_Expression(translate_input_symbol(yytext, qmath_lang)); return OP_EXP;}
<MATH>{OPERATOR_EQ}	{yylval=(int) new QMATH_Expression(translate_input_symbol(yytext, qmath_lang)); return OP_EQ;}
<MATH>{SEPARATOR_COMMA}	{yylval=(int) new QMATH_Expression(translate_input_symbol(yytext, qmath_lang)); return SEP_COMMA;}
<MATH>{SEPARATOR_SEMICOLON} {
                         yylval=(int) new QMATH_Expression(translate_input_symbol(yytext, qmath_lang)); return SEP_SCOLON;}
<MATH>{SEPARATOR_COLON}	{yylval=(int) new QMATH_Expression(translate_input_symbol(yytext, qmath_lang)); return SEP_COLON;}

<MATH>"("|"["		{
			  yylval=(int) new QMATH_Expression(yytext);
			  return FEN_OPEN;
			}
<MATH>")"|"]"		{
                          yylval=(int) new QMATH_Expression(yytext);
			  return FEN_CLOSE;
			}

<QMATH,MATH>\n\n	{
			  //if (inside_math)
			  //  {
			  //    end_math(); fprintf(yyout, "\n<p>\n\n"); begin_math();
			  //  }
			  //else
			  //  {
			      fprintf(yyout, "\n<p>\n\n");
			  //  }
			}
<QMATH,MATH>\n		{
			  //if (!inside_math) fprintf(yyout, "<br>");
			  fprintf(yyout, "%s", yytext);
			}





%%

int yywrap() {return 1;};
