%option noyywrap
%option nomain

	#include <stdio.h>
	#include <stdlib.h>
	#include <string.h>
	#include <math.h>
	#include <time.h>
	#include <locale.h>
	#include "actions.h"
	#include "qmath.tab.h"
	int yyparse(void);
	char* input_file_name = NULL;
        char* output_file_name = NULL;
        char* file_write_mode = NULL;
	#define MAX_BUFFER 255
	char buffer[MAX_BUFFER];
	#define MAX_SYSTEM_COMMAND (MAX_BUFFER + 256)
	char system_command[MAX_SYSTEM_COMMAND];
	int arg_count = 0;
	char* arg_count_postfix;
	int current_level = 0;
	int current_line = 1;
	#define MAX_LEVELS 200
	char* level_ending_tag[MAX_LEVELS];
	char* level_operator[MAX_LEVELS];
	char* level_operator2[MAX_LEVELS];
	int plot_current_level = 0;
	int table_column = 0;
	int table_row = 0;
	int current_part          = 0;
	int current_chapter       = 0;
	int current_section       = 0;
	int current_subsection    = 0;
	int current_subsubsection = 0;
	int current_definition    = 0;
	int current_proposition   = 0;
	int current_theorem       = 0;
        int current_equation      = 0;
	int current_figure        = 0;
	int html_part_heading = 1;
	int html_chapter_heading = 2;
	int html_section_heading = 3;
	int html_subsection_heading = 4;
	int html_subsubsection_heading = 5;
	int inside_math = 0;
	int inside_table = 0;
	int header_done = 0;
	void setmath(void);
	void sethtml(void);

INICIO_MATH			"$"
FIN_MATH			"$"
IDENTIFICADOR			[a-z]'*
CONJUNTO			[A-Z]'*
NUMERO				[0-9]+([.'][0-9]+)?
VECTOR				"_"[a-zA-Z]'*
FUNCION				[a-z]"("
IDENTIFICADOR_CUALQUIERA	{IDENTIFICADOR}|{CONJUNTO}|{VECTOR}
INICIO_NIVEL			"{"|"("|"["
FIN_NIVEL			"}"|")"|"]"

INDEX			"ndice("
BGCOLOR_TAG          \ BGCOLOR[ \t]*=[ \t]*\"[#0-9a-fA-F]+\"
TEXT_TAG             \ TEXT[ \t]*=[ \t]*\"[#0-9a-fA-F]+\"
BGPROPERTIES_TAG     \ BGPROPERTIES[ \t]*=[ \t]*\"[a-zA-Z]+\"

%x MATH
%x TABLE
%x PLOT

%%

<INITIAL,TABLE>{INICIO_MATH} {
			  setmath();
			  BEGIN(MATH);
			}
<MATH>{FIN_MATH}	{
			  sethtml();
			  if (inside_table) BEGIN(TABLE);
			  else              BEGIN(INITIAL);
			}
<MATH>","		{
			  if (current_level == 0) fprintf(yyout, "<mo>, </mo>");
			  else
			    {
			      if (arg_count > 0)
			        {
			          --arg_count;
			          if (arg_count == 0)
			            {
				      fprintf(yyout, "%s", arg_count_postfix);
			            }
			        }
			      fprintf(yyout, "%s", level_operator[current_level]);
			    }
			}
<MATH>";"		{
			  if (current_level == 0) fprintf(yyout, "<mo>; </mo>");
			  else
			    {
			      fprintf(yyout, "%s", level_operator2[current_level]);
			    }
			}
<MATH>{FUNCION}		{
			  ++current_level;
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "  </mrow>\n </mfenced>\n</mrow>\n";
			  strncpy(buffer, yytext, 1);
			  buffer[1] = 0;// Dejamos slo el nombre la funcin.
			  fprintf(yyout, "<mrow>\n <mi>%s</mi><mfenced open='(' close=')' separator=','>\n  <mrow>", buffer);
			}
<MATH>{IDENTIFICADOR}	{fprintf(yyout, "<mi>%s</mi>", yytext);}
<MATH>{NUMERO}		{yylval=(int)new_string(yytext); return NUM;/*fprintf(yyout, "<mn>%s</mn>", yytext);*/}
<MATH>"alpha"		{fprintf(yyout, "<mi> &alpha; </mi>");}
<MATH>"Alpha"		{fprintf(yyout, "<mi> &Alpha; </mi>");}
<MATH>"beta"		{fprintf(yyout, "<mi> &beta; </mi>");}
<MATH>"Beta"		{fprintf(yyout, "<mi> &Beta; </mi>");}
<MATH>"phi"		{fprintf(yyout, "<mi> &phiv; </mi>");}
<MATH>"Phi"		{fprintf(yyout, "<mi> &Phiv; </mi>");}
<MATH>"psi"		{fprintf(yyout, "<mi> &psi; </mi>");}
<MATH>"Psi"		{fprintf(yyout, "<mi> &Psi; </mi>");}
<MATH>"pi"		{fprintf(yyout, "<mi> &pi;  </mi>");}
<MATH>"Pi"		{fprintf(yyout, "<mi> &Pi;  </mi>");}
<MATH>"xi"		{fprintf(yyout, "<mi> &xi;  </mi>");}
<MATH>"Xi"		{fprintf(yyout, "<mi> &Xi;  </mi>");}
<MATH>"chi"		{fprintf(yyout, "<mi> &chi;  </mi>");}
<MATH>"Chi"		{fprintf(yyout, "<mi> &Chi;  </mi>");}
<MATH>"theta"		{fprintf(yyout, "<mi> &theta; </mi>");}
<MATH>"Theta"		{fprintf(yyout, "<mi> &Theta; </mi>");}
<MATH>"omega"		{fprintf(yyout, "<mi> &omega; </mi>");}
<MATH>"Omega"		{fprintf(yyout, "<mi> &Omega; </mi>");}
<MATH>"delta"		{fprintf(yyout, "<mi> &delta; </mi>");}
<MATH>"Delta"		{fprintf(yyout, "<mi> &Delta; </mi>");}
<MATH>""		{fprintf(yyout, "<mi> &deg; </mi>");}
<MATH>""		{fprintf(yyout, "<mo> &Not;</mo>");}
<MATH>"|"		{fprintf(yyout, "<mo> &Or; </mo>");}
<MATH>"&"		{fprintf(yyout, "<mo> &And; </mo>");}
<MATH>"->"		{fprintf(yyout, "<mo> &DoubleRightArrow; </mo>");}
<MATH>"<-"		{fprintf(yyout, "<mo> &DoubleLeftArrow; </mo>");}
<MATH>"<->"		{fprintf(yyout, "<mo> &DoubleLeftRightArrow; </mo>");}
<MATH>"ptodo"		{fprintf(yyout, "<mo> &ForAll; </mo>");}
<MATH>"existe"		{fprintf(yyout, "<mo> &Exists; </mo>");}
<MATH>"tq"		{fprintf(yyout, "<mo> | </mo>");}
<MATH>"="		{fprintf(yyout, "<mo> = </mo>");}
<MATH>"="		{fprintf(yyout, "<mo> &NotEqual; </mo>");}
<MATH>"+-"              {fprintf(yyout, "<mo> &PlusMinus; </mo>");}
<MATH>{CONJUNTO}	{fprintf(yyout, "<mi>%s</mi>", yytext);}
<MATH>"pert"		{fprintf(yyout, "<mo> &Element; </mo>");}
<MATH>"npert"		{fprintf(yyout, "<mo> &NotElement; </mo>");}
<MATH>"cont"		{fprintf(yyout, "<mo> &Superset; </mo>");}
<MATH>"icont"		{fprintf(yyout, "<mo> &SupersetEqual; </mo>");}
<MATH>"subcon"		{fprintf(yyout, "<mo> &Subset; </mo>");}
<MATH>"isubcon"		{fprintf(yyout, "<mo> &SubsetEqual; </mo>");}
<MATH>"unin"		{fprintf(yyout, "<mo> &Union; </mo>");}
<MATH>"inter"		{fprintf(yyout, "<mo> &Intersection; </mo>");}
<MATH>"vaco"		{sethtml(); fprintf(yyout, " &Oslash; "); setmath();}
<MATH>"\\"		{fprintf(yyout, "<mo> \\ </mo>");}
<MATH>"..."		{fprintf(yyout, "<mn> &TripleDot; </mn>");}

<MATH>"Prod("		{
			  ++current_level;
			  fprintf(yyout, "<mrow>");
			  level_operator[current_level] = "<mo></math> &times; <math></mo>";
			  level_ending_tag[current_level] = "</mrow>";
			}

<MATH>"~"		{fprintf(yyout, "<mo> &Congruent; </mo>");}
<MATH>">="		{fprintf(yyout, "<mo> &GreaterEqual; </mo>");}
<MATH>"<="		{fprintf(yyout, "<mo> &le; </mo>");}
<MATH>">"		{fprintf(yyout, "<mo> &gt; </mo>");}
<MATH>"<"		{fprintf(yyout, "<mo> &lt; </mo>");}
<MATH>"Inv("		{
			  ++current_level;
			  fprintf(yyout, "<msup><mrow>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow><mn>-1</mn></msup>";
			}
<MATH>""		{sethtml(); fprintf(yyout, "&bull;"); setmath();}

<MATH>"aleph"		{sethtml(); fprintf(yyout, "&alefsym;"); setmath();}
<MATH>"Intc("		{
			  ++current_level;
			  fprintf(yyout, "<mfenced open='[' close=']' separator=','>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfenced>";
			}
<MATH>"Inta("		{
			  ++current_level;
			  fprintf(yyout, "<mfenced open='(' close=')' separator=','>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfenced>";
			}
<MATH>"Intca("		{
			  ++current_level;
			  fprintf(yyout, "<mfenced open='[' close=')' separator=','>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfenced>";
			}
<MATH>"Intac("		{
			  ++current_level;
			  fprintf(yyout, "<mfenced open='(' close=']' separator=','>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfenced>";
			}
<MATH>"inf"		{sethtml(); fprintf(yyout, "&infin"); setmath();}

<MATH>"Conj("		{
			  ++current_level;
			  fprintf(yyout, "<mover><mrow>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow><mo>_</mo></mover>";
			}
<MATH>"Int("		{
			  ++current_level;
			  fprintf(yyout, "<mrow><msubsup><mo>&Integral;</mo>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow>";
			  arg_count = 2;
			  arg_count_postfix = "</msubsup>";
			}
<MATH>"Cont("		{
			  ++current_level;
			  fprintf(yyout, "<msup><mi>C</mi>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</msup>";
			}
<MATH>"Integ("		{
			  ++current_level;
			  fprintf(yyout, "<msup><mi>L</mi>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</msup>";
			}
<MATH>"prop"		{fprintf(yyout, "&Proportional;");}
<MATH>"Bin("		{
			  ++current_level;
			  fprintf(yyout, "<mfenced open='(' close=')'><mfrac>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfrac></mfenced>";
			}
<MATH>"Exp("		{
			  ++current_level;
			  fprintf(yyout, "<msup><mi>e</mi>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</msup>";
			}
<MATH>"Pot("		{
			  ++current_level;
			  fprintf(yyout, "<msup><mrow>");
			  level_operator[current_level] = "</mrow><mrow>";
			  level_ending_tag[current_level] = "</mrow></msup>";
			}
<MATH>"Raz("		{
			  ++current_level;
			  fprintf(yyout, "<msqrt><mrow>");
			  level_operator[current_level] = "</mrow><mrow>";
			  level_ending_tag[current_level] = "</mrow></msqrt>";
			}
<MATH>"Razn("		{
			  ++current_level;
			  fprintf(yyout, "<mroot><mrow>");
			  level_operator[current_level] = "</mrow><mrow>";
			  level_ending_tag[current_level] = "</mrow></mroot>";
			}
<MATH>"Weier("		{
			  sethtml(); fprintf(yyout, "&weierp;");
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mrow>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow>";
			}
<MATH>"Gamma("		{
			  sethtml(); fprintf(yyout, "&Gamma;");
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mrow>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow>";
			}
<MATH>"neutro"		{fprintf(yyout, "<mi>e</mi>");}
<MATH>"csuma"		{fprintf(yyout, "&CirclePlus;");}
<MATH>"cprod"		{fprintf(yyout, "&CircleTimes;");}
<MATH>{VECTOR}		{fprintf(yyout, "<mover><mi>%s</mi><mo>&RightArrow;</mo></mover>", yytext + 1);}
<MATH>"Norma("		{
			  ++current_level;
			  fprintf(yyout, "<mrow><mo>|</mo><mrow>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow><mo>|</mo></mrow>";
			}
<MATH>"T("		{
			  ++current_level;
			  fprintf(yyout, "<msup>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "<mi>t</mi></msup>";
			}
<MATH>"Det("		{
			  ++current_level;
			  fprintf(yyout, "<mrow><mo>|</mo><mrow>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow><mo>|</mo></mrow>";
			}
<MATH>"E("		{
			  ++current_level;
			  fprintf(yyout, "<msup><mi>E</mi>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</msup>";
			}
<MATH>"S("		{
			  ++current_level;
			  fprintf(yyout, "<msup><mi>S</mi>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</msup>";
			}
<MATH>"Dist("		{
			  ++current_level;
			  fprintf(yyout, "<mo>d</mo><mfenced open='(' close=')' separators=','>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfenced>";
			}
<MATH>"Mat("		{
			  ++current_level;
			  fprintf(yyout, "<mrow><mo>{</mo><mtable><mtr><mtd><mrow>");
			  level_operator[current_level] = "</mrow></mtd><mtd><mrow>";
			  level_operator2[current_level] = "</mrow></mtd></mtr><mtr><mtd><mrow>";
			  level_ending_tag[current_level] = "</mrow></mtd></mtr></mtable><mo>}</mo></mrow>";
			}


<MATH>"{"		{
			  ++current_level;
			  fprintf(yyout, "<mrow><mo>{</mo>\n");
			  level_operator[current_level]  = ",";
			  level_operator2[current_level] = ";";
			  level_ending_tag[current_level] = "<mo>}</mo></mrow>\n";
			}
<MATH>"\\{"		{
			  ++current_level;
			  fprintf(yyout, "<mrow>\n");
			  level_operator[current_level]  = ",";
			  level_operator2[current_level] = ";";
			  level_ending_tag[current_level] = "</mrow>\n";
			}
<MATH>"("		{
			  ++current_level;
			  fprintf(yyout, "<mrow><mo>(</mo>\n");
			  level_operator[current_level]  = ",";
			  level_operator2[current_level] = ";";
			  level_ending_tag[current_level] = "<mo>)</mo></mrow>\n";
			}
<MATH>"\\("		{
			  ++current_level;
			  fprintf(yyout, "<msup><mrow>\n");
			  level_operator[current_level] = "</mrow><mrow>";
			  level_operator2[current_level] = ";";
			  level_ending_tag[current_level] = "</mrow></msup>\n";
			}
<MATH>"/("		{
			  ++current_level;
			  fprintf(yyout, "<mfrac><mrow>");
			  level_operator[current_level] = "</mrow><mrow>";
			  level_operator2[current_level] = ";";
			  level_ending_tag[current_level] = "</mrow></mfrac>";
			}
<MATH>"["		{
			  ++current_level;
			  fprintf(yyout, "<mrow><mo>[</mo>\n");
			  level_operator[current_level] = "";
			  level_operator2[current_level] = ";";
			  level_ending_tag[current_level] = "<mo>]</mo></mrow>\n";
			}
<MATH>{IDENTIFICADOR_CUALQUIERA}"[" {
			  char* buffer;
			  buffer = calloc(strlen(yytext), sizeof(char));
			  strncpy(buffer, yytext, strlen(yytext));
			  buffer[strlen(yytext) - 1] = 0;
			  ++current_level;
			  fprintf(yyout, "<msub><mi>%s</mi><mrow>", buffer);
			  level_operator[current_level] = "</mrow><mrow>";
			  level_ending_tag[current_level] = "</mrow></msub>\n";
			  free(buffer);
			}
<MATH>{FIN_NIVEL}	{
			  if (current_level == 0)
			    {
			      fprintf(stderr, "Unmatched parentheses at line: %d\n", current_line);
			    }
			  else
			    {
			      fprintf(yyout, "%s", level_ending_tag[current_level]);
			      --current_level;
			    }
			}


"("			{
			  ++current_level;
			  fprintf(yyout, yytext);
			  level_operator[current_level]  = ",";
			  level_operator2[current_level] = ";";
			  level_ending_tag[current_level] = ")";
			}
")"			{
			  if (current_level == 0)
			    {
			      fprintf(stderr, "Unmatched parentheses at line: %d\n", current_line);
			    }
			  else
			    {
			      fprintf(yyout, "%s", level_ending_tag[current_level]);
			      --current_level;
			    }
			}
"Ttulo("[^)]+")"	{
			  current_part = 0;
			  current_chapter = 0;
			  current_section = 0;
			  current_subsection = 0;
			  current_subsubsection = 0;
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + strlen("Ttulo("));
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  fprintf(yyout, "<center><h1>%s</h1></center>", buffer);
			}
"Autor("[^)]+")"	{
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + strlen("Autor("));
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  fprintf(yyout, "Autor:<address>%s</address>", buffer);
			}
"Fecha("[^)]+")"	{
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + strlen("Fecha("));
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  if (strcmp("\auto", buffer))
			    {time_t tiempo; time(&tiempo); strftime(buffer, MAX_BUFFER, "%Y%m%d %X (%Z) %A", localtime(&tiempo));}
			  fprintf(yyout, "Fecha:<address>%s</address>", buffer);
			}
"Parte("[^)]+")"	{
			  ++current_part;
			  current_chapter = 0;
			  current_section = 0;
			  current_subsection = 0;
			  current_subsubsection = 0;
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + strlen("Parte("));
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  fprintf(yyout, "<a name='Part_%d_%s'></a><center><h%d>Parte %d.<br>%s</h%d></center><hr size=4>", current_part, buffer, html_part_heading, current_part, buffer, html_part_heading);
			}
"Captulo("		{
			  ++current_chapter;
			  current_section = 0;
			  current_subsection = 0;
			  current_subsubsection = 0;
			  fprintf(yyout, "<a name='Chapter_%d.%d'></a><center><h2>CAPTULO %d<br>", current_part, current_chapter, current_chapter);
			  ++current_level;
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</h2></center><hr>";
			}
"Seccin("		{
			  ++current_section;
			  current_subsection = 0;
			  current_subsubsection = 0;
			  fprintf(yyout, "<a name='Section_%d.%d.%d'></a><center><h3>%d.%d ", current_part, current_chapter, current_section, current_chapter, current_section);
			  ++current_level;
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</h3></center>";
			}
"Subseccin("[^)]+")"	{
			  ++current_subsection;
			  current_subsubsection = 0;
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + strlen("Subseccin("));
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  fprintf(yyout, "<a name='Subsection_%d.%d.%d.%d_%s'></a><center><h%d>%d.%d.%d %s</h%d></center>", current_part, current_chapter, current_section, current_subsection, buffer, html_subsection_heading, current_chapter, current_section, current_subsection, buffer, html_subsection_heading);
			}
"Subsubseccin("[^)]+")" {
			  ++current_subsubsection;
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + strlen("Subsubseccin("));
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  fprintf(yyout, "<a name='Subsubsection_%d.%d.%d.%d.%d_%s'></a><center><h%d>%d.%d.%d.%d %s</h%d></center>", current_part, current_chapter, current_section, current_subsection, current_subsubsection, buffer, html_subsubsection_heading, current_chapter, current_section, current_subsection, current_subsubsection, buffer, html_subsubsection_heading);
			}
"ndice("[^)]+")"	{
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + 7);
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  fprintf(yyout, "<a name=\"%s\"><dfn>%s</dfn></a>", buffer, buffer);
			}
"Def("[^,]*","			{
			  ++current_definition;
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + 4);
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  if (strlen(buffer) > 0)
			    {
			      fprintf(yyout, "<br>\n<a name='%s'></a><dfn>%s:", buffer, buffer);
			    }
			  else
			    {
			      fprintf(yyout, "<br>\n<a name='Definition_%d.%d'></a><dfn>Definicin %d:", current_chapter, current_definition, current_definition);
			    }
			  ++current_level;
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</dfn>\n";
			}
"Ec("			{
			  ++current_equation;
			  fprintf(yyout, "<br>\n<a name='Equation_%d.%d'></a><br>Ecuacin %d:<center>", current_chapter, current_equation, current_equation);
			  ++current_level;
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</center>\n";
			}
"Proposicin:"		{
			  ++current_proposition;
			  fprintf(yyout, "<br>\n<a name='Proposition_%d'><b>Proposicin %d:</b></a>", current_proposition, current_proposition);
			}
"Teorema:"		{
			  ++current_theorem;
			  fprintf(yyout, "<br>\n<a name='Teorema_%d'><b>Theorem %d:</b></a>", current_theorem, current_theorem);
			}
"Enlace("[^)]+")"	{
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + 7);
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  if (strrchr(buffer, '#'))
			    {
			      fprintf(yyout, "<a href=\"%s\">%s</a>", buffer, strrchr(buffer, '#') +1);
			    }
			  else
			    {
			      fprintf(yyout, "<a href=\"%s\">%s</a>", buffer, buffer);
			    }
			}
"Tabla:"		{
			  fprintf(yyout, "\n<table border='1' width='100%%'>");
			  table_column = 0;
			  table_row = 0;
			  inside_table = 1;
			  BEGIN(TABLE);
			}
<TABLE>[\t ]*"|"\n/[\t ]*("|"|"-") {
			  fprintf(yyout, " </td></tr>\n");
			  table_column = 0;
			  ++table_row;
			}
<TABLE>[\t ]*"|"\n	{
			  fprintf(yyout, " </td></tr>\n</table>\n");
			  inside_table = 0;
			  BEGIN(INITIAL);
			}
<TABLE>[\t ]*"|"(\t+|" "{2,}) {
			  if (table_column > 0)
			    {
			      fprintf(yyout, " </td><td align='right'> ");
			    }
			  else
			    {
			      fprintf(yyout, " <tr><td align='right'> ");
			    }
			  ++table_column;
			}
<TABLE>[\t ]*"|"/"|"	{
			  if (table_column > 0)
			    {
			      fprintf(yyout, " </td><td width='1'>");
			    }
			  else
			    {
			      fprintf(yyout, " <tr><td width='1'>");
			    }
			  ++table_column;
			}
<TABLE>[\t ]*"|"	{
			  if (table_column > 0)
			    {
			      fprintf(yyout, " </td><td> ");
			    }
			  else
			    {
			      fprintf(yyout, " <tr><td> ");
			    }
			  ++table_column;
			}
<TABLE>^[\t ]*"-"+\n/[\t ]*("|"|"-") // Rip off
<TABLE>^[\t ]*"-"+\n	{
			  fprintf(yyout, "</table>\n");
			  inside_table = 0;
			  BEGIN(INITIAL);
			}
<TABLE>^[\t ]*"-"+	{
			  fprintf(yyout, "</table><!-- Warning: Line should contain only '-' and RET -->\n");
			  inside_table = 0;
			  BEGIN(INITIAL);
			}
"Plot("			{
			  plot_current_level = 1;
			  sprintf(system_command, "echo 'set terminal png small color");
			  /*%s)' | gnuplot >figure_%d.png", buffer, current_figure);*/
			  BEGIN(PLOT);
			}
<PLOT>"("		{
			  ++plot_current_level;
			  yymore();
			}
<PLOT>")"		{
			  if (plot_current_level == 0)
			    {
			      fprintf(stderr, "Unmatched parentheses at line: %d\n", current_line);
			    }
			  else if (plot_current_level == 1)
			    {
			      sprintf(buffer, "\nplot %s", yytext);
			      buffer[strlen(buffer) - 1] = 0;
			      strcat(buffer, "'");
			      if (strlen(system_command) + strlen(buffer) < MAX_SYSTEM_COMMAND)
			        strcat(system_command, buffer);
			      else
			        fprintf(yyout, "<!-- ERROR:(%d) system command string too long: '%s%s' -->\n", current_line, system_command, buffer);
			      sprintf(buffer, " | gnuplot >figure_%d.png", current_figure);
			      if (strlen(system_command) + strlen(buffer) < MAX_SYSTEM_COMMAND)
			        strcat(system_command, buffer);
			      else
			        fprintf(yyout, "<!-- ERROR:(%d) system command string too long: '%s%s' -->\n", current_line, system_command, buffer);
			      fprintf(yyout, "<!-- system(\"%s\") -->\n", system_command);
			      system(system_command);
			      fprintf(yyout, "<img src='figure_%d.png'>\n", current_figure);
			      ++current_figure;
			      /*fprintf(yyout, "%s", level_ending_tag[current_level]);*/
			      --plot_current_level;
			      if (inside_table) BEGIN(TABLE);
			      else              BEGIN(INITIAL);
			    }
			  else
			    {
			      --plot_current_level;
			      yymore();
			    }
			}
<PLOT>""		{
			  if (plot_current_level == 1)
			    {
			      sprintf(buffer, "\nplot %s", yytext);
			      buffer[strlen(buffer) - 1] = 0;
			      strcat(buffer, "");
			      if (strlen(system_command) + strlen(buffer) < MAX_SYSTEM_COMMAND)
			        strcat(system_command, buffer);
			      else
			        fprintf(yyout, "<!-- ERROR:(%d) system command string too long: '%s%s' -->\n", current_line, system_command, buffer);
			    }
			  else yymore();
			}
<PLOT>[^)(]+		yymore();

\n\n			{
			  current_line += 2;
			  if (inside_math)
			    {
			      sethtml(); fprintf(yyout, "\n<p>\n\n"); setmath();
			    }
			  else
			    {
			      fprintf(yyout, "\n<p>\n\n");
			    }
			}
\n			{
			  ++current_line;
			  if (!inside_math) fprintf(yyout, "<br>");
			  fprintf(yyout, "%s", yytext);
			}





%%
void setmath(void)
  {
    if (!inside_math)
      {
        fprintf(yyout, "<math>");
	inside_math = 1;
      }
  }
void sethtml(void)
  {
    if (inside_math)
      {
        fprintf(yyout, "</math>");
	inside_math = 0;
      }
  }
int yywrap() {return 1;};

void print_help(char *command_name);
int main(int argc, char *argv[]);

int main(int argc, char *argv[])
  {
    setlocale(LC_ALL, "");
    file_write_mode = "wb";
    if (argc > 1)
      {
        int index;
        for (index = 1; index <argc; ++index)
          {
            if (strcmp(argv[index], "-h") == 0
                || strcmp(argv[index], "--help") == 0)
              {
                print_help(argv[0]);
                return(0);
              }
            else if (strcmp(argv[index], "-a") == 0)
              {
                file_write_mode = "ab";
              }
            else if (strcmp(argv[index], "-o") == 0)
              {
                if (argc > index) {
                  output_file_name = argv[index + 1];
                  ++index;
                }
              }
            else if (strcmp(argv[index], "-f") == 0)
              {
                if (argc > index) {
                  input_file_name = argv[index + 1];
                  ++index;
                }
              }
            else if (*argv[index] == '-')
              {
                fprintf(stderr, "Unknown option. Use %s -h for help.\n", argv[0]);
                return 1;
              }
            else if (!input_file_name)
              {
                input_file_name = argv[index];
              }
            else if (!output_file_name)
              {
                output_file_name = argv[index];
              }
            else
              {
                fprintf(stderr, "Too many inputs. Use %s -h for help.\n", argv[0]);
                return 1;
              }
          }
	if (input_file_name && output_file_name)
          {
            if (strcmp(input_file_name, output_file_name) == 0)
              {
                fprintf(stderr, "Can not write on input file.\n");
                return 1;
              }
          }
        if (input_file_name)
          {
            fprintf(stderr, "Processing file: %s\n", input_file_name);
            if (!(yyin = fopen( input_file_name, "rb" )))
              {
                fprintf(stderr, "Cannot open file for reading: %s\n", input_file_name);
                if (output_file_name) fclose(yyout);
                return 1;
              }
          }
        if (output_file_name)
          {
            if (!(yyout = fopen( output_file_name, file_write_mode )))
              {
                fprintf(stderr, "Cannot open file for writing: %s\n", output_file_name);
                return 1;
              }
          }
      }
    if (yyout) fprintf(yyout,  "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\"\n   \"http://www.w3.org/TR/REC-html40/loose.dtd\">\n<html><head><title>QMathML</title></head>\n<body>\n\n");
    else       fprintf(stdout, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\"\n   \"http://www.w3.org/TR/REC-html40/loose.dtd\">\n<html><head><title>QMathML</title></head>\n<body>\n\n");

    yyparse();//bison
    //yylex();//flex

    if (yyout) fprintf(yyout,  "\n\n</body>\n</html>");
    else       fprintf(stdout, "\n\n</body>\n</html>");
    fprintf(stderr, "Total source lines: %d\n", current_line);
    if (output_file_name) fclose(yyout);
    if (input_file_name) fclose(yyin);
    return 0;
  }

void print_help(char* command_name)
  {
    fprintf(stderr, "\n%s Written 1999 Alberto Gonzlez Palomo\n", command_name);
    fprintf(stderr, "Usage: %s [-a] [-f <input file>] [-o <output file>]\n", command_name);
    fprintf(stderr, "    -a   Adds the output to the end of <output file>\n");
    fprintf(stderr, "         The default is to overwrite <output file>\n\n");
    fprintf(stderr, "Note: input and output can not be the same file.\n");
    fprintf(stderr, "      If no input nor output file name is given, the program\n");
    fprintf(stderr, "      uses stdin or stdout, respectively.\n");
  }













