/* QMath mathematical notation system--QMath */

%{
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
void yyerror (char *s);
int yylex (void);
int no_main(void);
#include "QMATH_Expression.H"
#define YYSTYPE QMATH_Expression*
extern char* qmath_evaluator_module;
%}

/* BISON Declarations */
%token INT_NUM
%token FP_NUM
%token STRING
%token ID
%token FUNC
/* At first I used "OP_SUM" instead of "OP_PLUS", but for some
   reason Bison didn't like it.*/
%token OP_PLUS
%token OP_PROD
%token OP_EXP
%token OP_EQ
%token FEN_OPEN
%token SEP_COMMA
%token SEP_SCOLON
%token SEP_COLON
%token FEN_CLOSE

/* Precedence and associativity are determined here */
/* Less precedence */
%left SEP_COLON
%left SEP_SCOLON
%left SEP_COMMA
%right OP_EQ
%left OP_PLUS
%left OP_PROD
%right OP_EXP
/* More precedence */

/* Grammar follows */
%%
input:    /* empty string */
        | input line
;

line:     '\n'
        | exp '\n'  { QMATH_Expression* math_object;
                      math_object = new QMATH_Expression(1, $1);
                      math_object->Type(math)
                                 ->Select_evaluator(qmath_evaluator_module);
                      cout << endl << *(math_object) << endl;
                      delete math_object;
                    }
;

exp:      INT_NUM                { ($$ = $1)           ->Type(integer);      }
        | FP_NUM                 { ($$ = $1)           ->Type(fp_number);    }
        | STRING                 { ($$ = $1)           ->Type(char_string);  }
        | ID                     { ($$ = $1)           ->Type(variable);     }
        | OP_PLUS exp            {
          ($$ = new QMATH_Expression (2, $1->Type(symbol), $2))
            ->Type(application);                       }
        | exp OP_PLUS exp        {
          ($$ = new QMATH_Expression (3, $2->Type(symbol), $1, $3))
            ->Type(application);                       }
        | exp OP_PROD exp        {
          ($$ = new QMATH_Expression (3, $2->Type(symbol), $1, $3))
            ->Type(application);                       }
        | exp OP_EXP exp         {
          ($$ = new QMATH_Expression (3, $2->Type(symbol), $1, $3))
            ->Type(application);                       }
        | exp OP_EQ exp          {
          ($$ = new QMATH_Expression (3, $2->Type(symbol), $1, $3))
            ->Type(application);                       }
        | exp SEP_COLON exp      {
          ($$ = new QMATH_Expression (3, $2->Type(symbol), $1, $3))
            ->Type(application);                       }
        | exp SEP_SCOLON exp     {
          ($$ = new QMATH_Expression (3, $2->Type(symbol), $1, $3))
            ->Type(application);                       }
        | exp SEP_COMMA exp      {
          ($$ = new QMATH_Expression (3, $2->Type(symbol), $1, $3))
            ->Type(application);                       }
        | FEN_OPEN exp FEN_CLOSE { ($$ = $2)         /*->Type(application)*/;}
        | FUNC exp FEN_CLOSE     {
          ($$ = new QMATH_Expression (2, $1->Type(symbol), $2))
            ->Type(application);                       }
;
  
%%

#include <ctype.h>
#include <stdio.h>

void yyerror (char *s)  /* Called by yyparse on error */
{
  fprintf (stderr, "\nError: %s\n", s);
}
