#include "QMATH_Evaluator_loader.H"

#include "QMATH_Evaluator.H"

extern QMATH_Evaluator_loader evaluator_loader;

//public:
QMATH_Evaluator::QMATH_Evaluator ()
{
  Select("default");
}

QMATH_Evaluator::QMATH_Evaluator (c_string evaluator_class)
{
  Select(evaluator_class);
}

void QMATH_Evaluator::Select (c_string evaluator_class)
{
  if (!evaluator_class) return;
  evaluator_loader.Get_evaluator_entry(selected_evaluator_iterator, evaluator_class);
}


// Last resort for broken evaluators: just output the list contents.
void Emergency_default (QMATH_Expression &exp, evaluator_table_entry &selected_evaluator)
{
  cerr << "The selected QMath evaluator " << selected_evaluator.name << " doesn't have a default operator." << endl
       << "Using emergency default operator instead." << endl
       << "Please report this bug to the QMath evaluator's maintainer." << endl;
  
  switch (exp.Type())
    {
    case none:         cout << "NONE.";     break;
    case integer:      cout << "INT.";      break;
    case fp_number:    cout << "FP.";       break;
    case char_string:  cout << "STR.";      break;
    case byte_array:   cout << "ARRAY.";    break;
    case symbol:       cout << "SYM.";      break;
    case variable:     cout << "VAR.";      break;
    case application:  cout << "APP ";      break;
    case attribution:  cout << "ATT ";      break;
    case binding:      cout << "BIND ";     break;
    case error:        cout << "ERR ";      break;
    default:           cout << "UNKNOWN"; break;
    }
  
  if (exp.Object_count() == 0)
    {
      cout << exp.Basic_object() << " ";
    }
  else
    {
      cout << "( ";
      for (int index = 0; index < exp.Object_count(); ++index)
        {
          exp.Object(index).Evaluate();
        }
      cout << ") ";
    }
}

// Apply evaluator.
void QMATH_Evaluator::Apply (QMATH_Expression &exp)
{
  list<operator_table_entry>::iterator operator_table_iterator;

  if (exp.Object_count() == 0)
    {//    If exp is a basic object, evaluate as the operator "default", with
      // no arguments.
      operator_table_iterator = Find_operator(string("default"));
      if (operator_table_iterator != selected_evaluator_iterator->operator_table.end())
        (*operator_table_iterator).code(exp);
      else
        //  The selected evaluator is a broken one.
        Emergency_default(exp, *selected_evaluator_iterator);

      return;
    }
  
  string operator_name;
  if (exp.Type() == math)
    {
      operator_name = "math";// We got a top level math object.
    }
  else
    {
      if (!exp.Object(0).Symbol(&operator_name))
        {// TODO: remove this part. This should be an 'impossible' condition.
          if (exp.Object(0).Basic_object())
            cout << "[" << exp.Object(0).Basic_object() << "] " ;
          else
            {
              cout << "{----";
              exp.Object(0).Evaluate();
              cout << "----}";
            }
          return;
        }
    }
  
  operator_table_iterator = Find_operator(operator_name);
  if (operator_table_iterator != selected_evaluator_iterator->operator_table.end())
    (*operator_table_iterator).code(exp);
  else
    {// If couldn't find the requested operator, look for the default one.
      operator_table_iterator = Find_operator(string("default"));
      if (operator_table_iterator != selected_evaluator_iterator->operator_table.end())
        (*operator_table_iterator).code(exp);
      else
        //  The selected evaluator is a broken one.
        Emergency_default(exp, *selected_evaluator_iterator);
    }
}

//private:
list<operator_table_entry>::iterator QMATH_Evaluator::Find_operator (string operator_name)
{
  list<operator_table_entry>::iterator operator_table_iterator;
  
  for (operator_table_iterator =  selected_evaluator_iterator->operator_table.begin();
       operator_table_iterator != selected_evaluator_iterator->operator_table.end();
       ++operator_table_iterator)
    if (operator_table_iterator->name.compare(operator_name) == 0)
      break;
  return operator_table_iterator;
}
