#include "QMATH_Evaluator_loader.H"

!@evaluator_list = ( "default", "mathml" );

!foreach $evaluator (@evaluator_list) {
#include "evaluators/Evaluator_$evaluator.H"
!}

QMATH_Evaluator_loader evaluator_loader;

//public:
QMATH_Evaluator_loader::QMATH_Evaluator_loader()
{
  // Clear previous evaluator table.
  evaluator_table.erase(evaluator_table.begin(), evaluator_table.end());
  
  evaluator_table_entry evaluator_entry;

!foreach $evaluator (@evaluator_list) {
  //----------------------------------------------
  evaluator_entry.operator_table.erase(evaluator_entry.operator_table.begin(), evaluator_entry.operator_table.end());
  Evaluator_${evaluator}_Register(evaluator_entry);
  evaluator_table.push_back(evaluator_entry);
!}

  Print_evaluator_table("QMATH_Evaluator_loader()");
}

void QMATH_Evaluator_loader::Get_evaluator_entry(list<evaluator_table_entry>::iterator &evaluator_iterator, c_string evaluator_class)
{
  list<evaluator_table_entry>::iterator selected_evaluator_iterator;

  Print_evaluator_table("Get_evaluator_entry()");
  for (selected_evaluator_iterator = evaluator_table.begin();
       selected_evaluator_iterator != evaluator_table.end();
       ++selected_evaluator_iterator)
    if (selected_evaluator_iterator->name.compare(evaluator_class) == 0)
      {
        break;
      }
  
  if (selected_evaluator_iterator == evaluator_table.end())
    {
      //  If we were asked for the default evaluator, and failed,
      // just exit with no action to avoid infinite recursion.
      //  This shouldn't happen, as the default evaluator must be
      // always available.
      if (strcmp("default", evaluator_class) == 0) return;

      // If the requested evaluator isn't available, use the default.
      Get_evaluator_entry(evaluator_iterator, "default");
      cerr << "Requested evaluator '" << evaluator_class << "' not found." << endl << "Using default evaluator instead." << endl;
    }
  else
    evaluator_iterator = selected_evaluator_iterator;
}

void QMATH_Evaluator_loader::Print_evaluator_table (c_string func_name)
{
  return;// Do nothing usually. If in trouble, comment out this line.
  cerr << endl << "begin list: " << func_name << endl;
  cerr << "This = " << this << endl;
  list<evaluator_table_entry>::iterator evaluator_table_print_iterator;
  evaluator_table_print_iterator = evaluator_table.begin();
  while (evaluator_table_print_iterator != evaluator_table.end())
    {
      cerr << "Evaluator table entry:" << evaluator_table_print_iterator->name << endl;
      list<operator_table_entry>::iterator operator_table_print_iterator;
      operator_table_print_iterator = evaluator_table_print_iterator->operator_table.begin();
      while (operator_table_print_iterator != evaluator_table_print_iterator->operator_table.end())
        {
          cerr << operator_table_print_iterator->name << ", ("
               << operator_table_print_iterator->code << ")"
               << endl;
          ++operator_table_print_iterator;
        }
      ++evaluator_table_print_iterator;
    }
  cerr << "end list" << endl << endl;
}
